@@ -7,7 +7,7 @@ use warnings;
 use Carp         'confess';
 use Scalar::Util 'weaken';
 
-our $VERSION   = '0.71_02';
+our $VERSION   = '0.77_01';
 $VERSION = eval $VERSION;
 our $AUTHORITY = 'cpan:STEVAN';
 
@@ -64,8 +64,6 @@ sub _new {
 
 ## accessors
 
-sub body { (shift)->{'body'} }
-
 sub associated_metaclass { shift->{'associated_metaclass'} }
 
 sub attach_to_class {
@@ -79,10 +77,6 @@ sub detach_from_class {
     delete $self->{associated_metaclass};
 }
 
-sub package_name { (shift)->{'package_name'} }
-
-sub name { (shift)->{'name'} }
-
 sub fully_qualified_name {
     my $self = shift;
     $self->package_name . '::' . $self->name;
@@ -168,10 +162,8 @@ to this class.
 =item B<wrap ($code, %params)>
 
 This is the basic constructor, it returns a B<Class::MOP::Method>
-instance which wraps the given C<$code> reference. You can also
-set the C<package_name> and C<name> attributes using the C<%params>.
-If these are not set, then thier accessors will attempt to figure
-it out using the C<Class::MOP::get_code_info> function.
+instance which wraps the given C<$code> reference. You must also set
+the C<package_name> and C<name> attributes in C<%params>.
 
 =item B<clone (%params)>
 
